
# q-expansion of newform 3600.3.j.h, downloaded from the LMFDB on 02 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3600
weight = 3
poly_data = [1, 0, -1, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 0, 0, 1], 1], [[-1, 0, 2, 0], 1], [[0, 2, 0, -1], 1]]

hecke_ring_character_values = [[3151, [-1, 0, 0, 0]], [901, [1, 0, 0, 0]], [2801, [1, 0, 0, 0]], [577, [-1, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, -1], [0, 0, 2, 0], [0, 11, 0, 0], [0, -6, 0, 0], [0, 0, -11, 0], [0, 0, 0, -10], [30, 0, 0, 0], [0, 0, -3, 0], [0, 14, 0, 0], [-36, 0, 0, 0], [0, 0, 0, -3], [0, 0, 0, 26], [0, -72, 0, 0], [0, 0, 22, 0], [35, 0, 0, 0], [0, 0, 0, -17], [0, 0, -52, 0], [0, 62, 0, 0], [0, 0, -44, 0], [0, 0, 0, 42], [144, 0, 0, 0], [0, 181, 0, 0], [48, 0, 0, 0], [0, 0, 0, -56], [0, 0, 0, 104], [-169, 0, 0, 0], [0, -12, 0, 0], [0, 0, 0, 60], [0, 0, -40, 0], [0, -54, 0, 0], [0, 0, -28, 0], [234, 0, 0, 0], [0, 0, 95, 0], [0, 35, 0, 0], [0, 0, 0, -101], [0, 0, 0, 80], [0, -198, 0, 0], [0, 0, -146, 0], [-23, 0, 0, 0], [0, 0, -138, 0], [0, -59, 0, 0], [0, 162, 0, 0], [0, 0, -127, 0], [0, 0, -233, 0], [0, 0, 0, 129], [0, 0, 0, 244], [59, 0, 0, 0], [0, -84, 0, 0], [0, 0, -112, 0], [73, 0, 0, 0], [0, 0, 20, 0], [0, 228, 0, 0], [0, 0, 0, -244], [-270, 0, 0, 0], [0, 0, 20, 0], [0, 49, 0, 0], [246, 0, 0, 0], [0, 0, 0, -81], [0, 78, 0, 0], [0, 0, 0, 117], [0, 0, -214, 0], [0, 25, 0, 0], [0, 600, 0, 0], [0, 0, -236, 0], [0, -203, 0, 0], [0, 0, 0, 354], [538, 0, 0, 0], [0, 498, 0, 0], [0, 0, -52, 0], [0, 0, 0, 329], [0, -491, 0, 0], [0, 0, 187, 0], [0, 0, 0, -212], [-132, 0, 0, 0], [0, 251, 0, 0], [720, 0, 0, 0], [515, 0, 0, 0], [0, 0, -456, 0], [-278, 0, 0, 0], [0, 0, -362, 0], [0, -287, 0, 0], [0, 0, -141, 0], [0, 0, 0, 128], [132, 0, 0, 0], [0, -434, 0, 0], [-312, 0, 0, 0], [0, 0, 0, 52], [0, 0, 0, -2], [0, 0, -426, 0], [0, 0, 0, 149], [0, 0, 192, 0], [0, 0, 177, 0], [0, 0, 0, -108], [-546, 0, 0, 0], [318, 0, 0, 0], [0, 0, 0, -517], [157, 0, 0, 0], [0, 0, 0, -40], [0, -114, 0, 0], [0, 0, 0, -74], [450, 0, 0, 0], [0, 0, -13, 0], [0, 311, 0, 0], [0, 0, 0, 96], [0, 636, 0, 0], [0, 0, 444, 0], [371, 0, 0, 0], [0, 0, 0, 308], [0, -374, 0, 0], [0, -240, 0, 0], [0, 0, -359, 0], [0, 0, 317, 0], [300, 0, 0, 0], [0, 0, 0, 228], [0, 0, 0, 404], [0, -534, 0, 0], [0, 0, 252, 0], [-34, 0, 0, 0], [0, 1234, 0, 0], [0, -534, 0, 0], [0, 0, 0, 304], [0, 0, -648, 0], [-306, 0, 0, 0], [-433, 0, 0, 0], [0, 0, 338, 0], [0, 0, 0, -537], [0, 590, 0, 0], [0, 0, 372, 0], [0, 0, 0, 428], [0, 0, 524, 0], [0, 841, 0, 0], [-924, 0, 0, 0], [-347, 0, 0, 0], [0, -1404, 0, 0], [0, 0, 0, 25], [0, -348, 0, 0], [384, 0, 0, 0], [0, 0, 399, 0], [-462, 0, 0, 0], [0, 0, 0, -355], [0, 0, 0, 112], [-1606, 0, 0, 0], [0, 0, 518, 0], [0, -575, 0, 0], [0, -1068, 0, 0], [0, 0, -712, 0], [0, 0, 0, -96], [0, 731, 0, 0], [-48, 0, 0, 0], [0, 0, 0, -599], [0, 0, 0, -794], [0, 0, 0, -884], [0, 0, 802, 0], [0, 0, 81, 0], [1002, 0, 0, 0], [0, -661, 0, 0], [-1110, 0, 0, 0], [0, 0, 0, -582], [0, 1680, 0, 0], [0, 0, 0, -356], [0, 0, -178, 0], [0, -1578, 0, 0], [0, 0, 0, 796], [0, 0, -47, 0], [0, -34, 0, 0], [383, 0, 0, 0], [0, -78, 0, 0], [0, 0, 438, 0], [-1045, 0, 0, 0], [0, 0, -14, 0], [0, 1583, 0, 0], [0, 0, 953, 0], [-168, 0, 0, 0], [0, 0, 267, 0], [-114, 0, 0, 0], [0, 0, 0, -907], [493, 0, 0, 0], [0, 0, 0, -281], [0, 0, 328, 0], [0, -337, 0, 0], [0, -2034, 0, 0], [0, 0, 0, -210], [-1362, 0, 0, 0], [0, -1583, 0, 0], [0, 0, 0, 1135], [-169, 0, 0, 0], [0, 0, 922, 0], [0, -1921, 0, 0], [0, 0, 0, 412], [0, 0, 697, 0], [-930, 0, 0, 0], [0, 0, 0, -1226], [0, 1566, 0, 0], [-178, 0, 0, 0], [0, -10, 0, 0], [0, 654, 0, 0], [0, 0, 0, 774], [-2052, 0, 0, 0], [0, 0, -808, 0], [0, -1082, 0, 0], [1502, 0, 0, 0], [0, 0, 1340, 0], [0, -606, 0, 0], [0, 0, 1309, 0], [0, 0, 0, -470], [-2100, 0, 0, 0], [0, 0, 329, 0], [0, -325, 0, 0], [-906, 0, 0, 0], [0, 0, 0, 1313], [0, 0, 0, -988], [0, 0, -206, 0], [1334, 0, 0, 0], [0, 0, 0, -621], [1074, 0, 0, 0], [0, 0, 0, -436], [0, -1854, 0, 0], [313, 0, 0, 0], [0, 0, -627, 0], [-78, 0, 0, 0], [0, 0, 0, -564], [0, 0, 0, -812], [2629, 0, 0, 0], [0, 1614, 0, 0], [0, 0, -1190, 0], [0, 0, 0, -919], [0, 0, -486, 0], [0, -2725, 0, 0], [0, 0, 1428, 0], [0, 0, -1233, 0], [2058, 0, 0, 0], [0, 0, 0, 796], [0, 0, 0, -512], [758, 0, 0, 0], [0, 216, 0, 0], [0, 0, 354, 0], [0, 0, 1216, 0], [0, 0, 0, -678], [0, 0, 1061, 0], [0, 0, 0, -1165], [157, 0, 0, 0], [0, 1974, 0, 0], [0, 0, 814, 0], [0, 0, 0, 973], [0, 0, -86, 0], [0, 0, 291, 0], [0, 0, 0, 178], [0, 1354, 0, 0], [-318, 0, 0, 0], [0, 0, 0, 692], [-289, 0, 0, 0], [0, 942, 0, 0], [0, 0, -916, 0], [-1151, 0, 0, 0], [0, 0, 0, -636], [0, 2952, 0, 0], [0, -10, 0, 0], [0, 0, 0, 1525], [0, 0, 0, -774], [-274, 0, 0, 0], [0, -3119, 0, 0], [0, -450, 0, 0], [0, 0, -1152, 0], [-1686, 0, 0, 0], [-1464, 0, 0, 0], [0, 0, 0, -1692], [0, -2622, 0, 0], [1703, 0, 0, 0], [0, 0, 0, 368], [0, 1550, 0, 0], [0, 0, 1255, 0], [0, 1273, 0, 0], [0, 0, 0, -1799], [0, 0, 0, 606], [-1175, 0, 0, 0], [-865, 0, 0, 0], [0, 0, -464, 0], [0, 0, 0, 1780], [0, 0, -1284, 0], [0, 0, 0, 854], [1475, 0, 0, 0], [0, 0, 0, 917], [0, 0, -794, 0], [0, 1103, 0, 0], [0, -2220, 0, 0], [0, 0, -1852, 0], [1764, 0, 0, 0], [-978, 0, 0, 0], [0, 0, 0, 948], [0, 2004, 0, 0], [0, 0, -1360, 0], [0, 3109, 0, 0], [780, 0, 0, 0], [0, 0, -1487, 0], [0, -3300, 0, 0], [0, 0, 1542, 0], [0, 0, 0, 1396], [0, -1550, 0, 0], [0, -3300, 0, 0], [0, 0, 1407, 0], [0, 0, 0, -890], [0, 0, 796, 0], [0, 2773, 0, 0], [0, 0, 0, 2020], [-733, 0, 0, 0], [0, 0, 1280, 0], [0, -2186, 0, 0], [0, 0, 0, -774], [2148, 0, 0, 0], [684, 0, 0, 0], [0, 0, 0, -851], [0, 0, 0, 2106], [-325, 0, 0, 0], [0, 0, -1386, 0], [0, 0, 476, 0], [0, 2435, 0, 0], [2682, 0, 0, 0], [0, 0, -1480, 0], [0, -2257, 0, 0], [-480, 0, 0, 0], [0, 0, 0, -1851], [0, 786, 0, 0], [3442, 0, 0, 0]]
